#!/bin/bash
#
# 08/10/2013 Updated to add detection for tomcat verstion (RA)
# 14/02/2014 Updated to copy cache.xml from old location to new config dir (RA)

source /backups/patch/settings.sh

cleanup() {
 cd $PATCHBASE
 rm -rf *
 rm -rf /tmp/pinsafe
 echo; echo "Clean up done..."
}

initialise() {
 if test -f /etc/rc.d/init.d/tomcat5; then
  TOMCAT=tomcat5; fi
 if test -f /etc/rc.d/init.d/tomcat; then
  TOMCAT=tomcat; fi
 if test ! $TOMCAT; then
  echo "Unable to detect the vesion of tomcat installed."
  echo "Exiting..."
  exit; fi
}

appliance_type() {
 PINSAFE_TYPE=`grep PINSAFE_TYPE $PINSAFE_CONF|cut -d\= -f2`

 if test ! $PINSAFE_TYPE; then
  stnd_err "Appliance type is not correctly configured."; fi
 if test "$PINSAFE_TYPE" = "Single"; then
  PINSAFE_TYPE="Single "; fi
}

check_drbd() {
 DRBD_TYPE=`grep DRBD $PINSAFE_CONF|cut -d\= -f2`
 
 if test "$DRBD_TYPE" = "yes"; then
   SWIVEL=$SWIVEL_DRBD
 fi
}

heading() {
 clear
 echo "${inv} Swivel Maintenance (c) $COPYRIGHT_YEAR${nor}                                             ${inv}$PINSAFE_TYPE${nor}"
 echo
 echo $1
 echo
}

yorn() {
 echo
 echo -n "Are you sure? (Yes/No): "
 read yn
 check_ans $1 $yn
}

check_ans() {
 if test ! $2; then
  $1; fi
 if test ! $2 = "Yes" && test ! $2 = "yes" && test ! $2 = "y" && test ! $2 = "Y" ; then
  $1; fi
}

check_running() {
 if test "`ps -eaf|grep $2|egrep -v "UID|grep|mysqld_safe|alert.d" 2>/dev/null`"; then
  if test ! -f $LOCK/$1; then
   touch $LOCK/$1; fi
 else
  if test -f $LOCK/$1; then
   rm -f $LOCK/$1; fi
 fi

 if test -f $LOCK/$1; then
  value="Running"
 else
  value="Stopped"
 fi
}

show_dots() {
 i=0
 while test $i -lt $1; do
  echo -n "."
  sleep 1 
  ((i = i + 1))
 done
}

service_sands_tomcat() {
 nohup service $TOMCAT $1 > /dev/null 2>&1 &
 echo -n "Tomcat: $1"
 show_dots 4
}

service_sands() {
 service $1 $2
}

warning() {
 heading "Upgrading Swivel"
 echo "Continuing will apply patch ${inv}Swivel $PATCH_VER${nor}."
 echo "The Swivel solution cannot be used whilst upgrading."
 echo
 yorn exit
}

appliance_version() {
 APPLIANCE_VERS=`grep ^APPLIANCE_VERS= $PINSAFE_CONF|cut -d\= -f2`
 APPLIANCE_VERS=$(echo $APPLIANCE_VERS|awk -F\. '{print $1$2$3}')
 APPLIANCE_REQUIRED=$(echo $APPLIANCE_REQUIRED|awk -F\. '{print $1$2$3}')

 if test $APPLIANCE_VERS && test $APPLIANCE_REQUIRED; then
  if test $APPLIANCE_VERS -lt $APPLIANCE_REQUIRED; then
   echo "The appliance requires patching before Swivel can be updated"
   echo; read -p "Press Return to exit" yn
   exit; fi
 else
  echo "Unable to determine appliance version"
  echo; read -p "Press Return to exit" yn
  exit
 fi
}

update_values() {
 sed -i "s/PINSAFE_PATCH_VER=/PINSAFE_PATCH_VER=$PATCH_VER/g" $PINSAFE_CONF 
}

update_firewall() {
 if test ! "`grep $SWIVEL_PORT $IPTABLES`"; then
  echo "Updating Appliance firewall to enable Swivel synchronisation"
  sed -i '/--dport 10000/a-A RH-Firewall-1-INPUT -m state --state NEW -m tcp -p tcp --dport 61616 -j ACCEPT' $IPTABLES
 # filesize=`cat $IPTABLES|wc -l`
#  ((linestoextract = $filesize -2))

#  head -$linestoextract $IPTABLES>/tmp/iptables.$$
# echo "-A RH-Firewall-1-INPUT -m state --state NEW -m tcp -p tcp --dport $SWIVEL_PORT -j ACCEPT">>/tmp/iptables.$$
# tail -2 $IPTABLES>>/tmp/iptables.$$
# cat /tmp/iptables.$$ > $IPTABLES
# rm /tmp/iptables.$$
  service iptables restart
 fi
}

install_swivel() {
 check_running $TOMCAT tomcat
 if test $value = "Running"; then
  echo "Stopping Tomcat and Swivel service"
  service_sands $TOMCAT stop
 fi
 
 echo "Creating new Swivel configuration dir."
 mkdir -p $SWIVEL
 chown -R swivel:swivel $SWIVEL
 rm -f /opt/.swivel
 ln -s /home/swivel/.swivel /opt/.swivel
 sed -i '/SWIVEL_HOME/cexport SWIVEL_HOME=/home/swivel/.swivel' /etc/profile.d/swivel.sh
 export SWIVEL_HOME=/home/swivel/.swivel

 echo "Creating backups of original configurations"
 if test -d $TOMCAT_PATH/webapps/pinsafe/WEB-INF/conf; then
  cp -rp $TOMCAT_PATH/webapps/pinsafe/WEB-INF/conf $SWIVEL; fi
 if test -d $TOMCAT_PATH/webapps/pinsafe/WEB-INF/data; then
  cp -rp $TOMCAT_PATH/webapps/pinsafe/WEB-INF/data $SWIVEL; fi
 if test -d $TOMCAT_PATH/webapps/pinsafe/WEB-INF/db; then
  cp -rp $TOMCAT_PATH/webapps/pinsafe/WEB-INF/db $SWIVEL; fi
 if test -d $TOMCAT_PATH/webapps/pinsafe/WEB-INF/logs; then
  cp -rp $TOMCAT_PATH/webapps/pinsafe/WEB-INF/logs $SWIVEL; fi
 if test -d $TOMCAT_PATH/webapps/pinsafe/WEB-INF/active-mq; then
  cp -rp $TOMCAT_PATH/webapps/pinsafe/WEB-INF/active-mq $SWIVEL; fi
 if test -d $TOMCAT_PATH/webapps/pinsafe/WEB-INF/reports; then
  cp -rp $TOMCAT_PATH/webapps/pinsafe/WEB-INF/reports $SWIVEL; fi
 if test -f $TOMCAT_PATH/webapps/pinsafe/WEB-INF/classes/cache.xml; then
  cp -rp $TOMCAT_PATH/webapps/pinsafe/WEB-INF/classes/cache.xml $SWIVEL/conf; fi

 echo "Creating backup of original Swivel to /tmp"
 if test -d $TOMCAT_PATH/webapps/pinsafe; then
  cp -rp $TOMCAT_PATH/webapps/pinsafe /tmp; fi

 echo "Starting Tomcat and Swivel service"
 service_sands_tomcat start

 echo; echo "Removing old PINsafe instance, please wait..."
 rm -f $TOMCAT_PATH/webapps/pinsafe.war
 sleep 20
 if test -d $TOMCAT_PATH/webapps/pinsafe; then
  rm -rf $TOMCAT_PATH/webapps/pinsafe; fi 

 echo; echo "Deploying new Swivel software"
 cp $PATCHBASE/pinsafe.war $TOMCAT_PATH/webapps 

 echo "Upgrade complete"
}

appliance_type
heading "Upgrading Swivel"
initialise
appliance_version
check_drbd
warning
install_swivel
update_values
update_firewall
cleanup
echo; read -p "Press Return to continue" yn
